package net.jieyuu.controller;


import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import net.jieyuu.enums.BizCodeEnum;
import net.jieyuu.request.UserLoginRequest;
import net.jieyuu.request.UserRegisterRequest;
import net.jieyuu.service.AddressService;
import net.jieyuu.service.FileService;
import net.jieyuu.service.NotifyService;
import net.jieyuu.service.UserService;
import net.jieyuu.utils.CommonUtil;
import net.jieyuu.utils.JsonData;
import net.jieyuu.vo.UserVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpRequest;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author jieyuu
 * @since 2024-02-12
 */
@Api(tags = "用户模块")
@RestController
@RequestMapping("/api/user/v1/")
public class UserController {
    @Autowired
    AddressService addressService;
    @Autowired
    FileService fileService;
    @Autowired
    UserService userService;
    @Autowired
    private NotifyService notifyService;

    @ApiOperation("根据id查询地址详情")
    @GetMapping("detail/{address_id}")
    public Object detail(
            @ApiParam(value = "地址", required = true)
            @PathVariable("address_id") long id) {
        return addressService.detail(id);
    }

    /**
     * 用户头像上传
     * 默认最大1M,超过报错
     *
     * @return
     */
    @ApiOperation("用户头像上传")
    @PostMapping(value = "upload")
    public JsonData uploadUserImg(
            @ApiParam(value = "文件上传", required = true)
            @RequestPart("file") MultipartFile file) throws IOException {
        String result = fileService.uploadUserHeadImg(file);

        return result != null ? JsonData.buildSuccess(result) : JsonData.buildResult(BizCodeEnum.FILE_UPLOAD_USER_IMG_FAIL);
    }

    /**
     * 用户注册
     *
     * @param registerRequest
     * @return
     */
    @ApiOperation("用户注册")
    @PostMapping(value = "register")
    public JsonData register(
            @ApiParam(value = "用户注册对象", required = true)
            @RequestBody UserRegisterRequest registerRequest) {
        JsonData jsonData = userService.register(registerRequest);

        return jsonData;
    }

    /**
     * 登录
     *
     * @param loginRequest
     * @return
     */
    @ApiOperation("用户登录")
    @PostMapping("login")
    public JsonData userLogin(@ApiParam("用户登陆对象") @RequestBody UserLoginRequest loginRequest) {

        JsonData jsonData = userService.login(loginRequest);
        return jsonData;
    }


//    @ApiOperation("刷新accessToken")
//    @PostMapping("refresh")
//    public JsonData getRefreshToken(Map<String, Object> param) {
//        //先从redis寻找refresh_token
//        //refresh_token存在, 解密accessToken
//        //重新调用JWTUtil 重新生成token
//
//        return null;
//        //不存在 重新登录
//    }


    /**
     * 个人信息查询
     * @return
     */
    @ApiOperation("获取用户信息")
    @GetMapping("detail")
    public JsonData getUserInfo() {
        UserVO userVO = userService.findUserDetail();
        return JsonData.buildSuccess(userVO);
    }

}

